#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include "SDL.h"

using namespace std;

#include "ScreenUtilities.h"
#include "consts.h"
#include "SDL_gfxPrimitives.h"


		void ScreenUtilities::drawSprite(SDL_Surface* imageSurface, SDL_Surface* screenSurface, int srcX, int srcY, int dstX, int dstY, int width, int height)
		{
			SDL_Rect srcRect;
			srcRect.x = srcX;
			srcRect.y = srcY;
			srcRect.w = width;
			srcRect.h = height;

			SDL_Rect dstRect;  
			dstRect.x = dstX;
			dstRect.y = dstY;
			dstRect.w = width;  
			dstRect.h = height; 
		   
			SDL_BlitSurface(imageSurface, &srcRect, screenSurface, &dstRect);
		}

		void ScreenUtilities::PutPixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
		{
			int bpp = surface->format->BytesPerPixel;
			Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

			switch(bpp) 
			{
				case 1:
					*p = pixel;
					break;

				case 2:
					*(Uint16 *)p = pixel;
					break;

				case 3:
					if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
						p[0] = (pixel >> 16) & 0xff;
						p[1] = (pixel >> 8) & 0xff;
						p[2] = pixel & 0xff;
					} else {
						p[0] = pixel & 0xff;
						p[1] = (pixel >> 8) & 0xff;
						p[2] = (pixel >> 16) & 0xff;
					}
					break;

				case 4:
					*(Uint32 *)p = pixel;
					break;
			}
		}



		Uint32 ScreenUtilities::GetPixel(SDL_Surface *surface, int x, int y)
		{
			int bpp = surface->format->BytesPerPixel;
			Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

			switch(bpp) {
			case 1:
				return *p;

			case 2:
				return *(Uint16 *)p;

			case 3:
				if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
					return p[0] << 16 | p[1] << 8 | p[2];
				else
					return p[0] | p[1] << 8 | p[2] << 16;

			case 4:
				return *(Uint32 *)p;

			default:
				return 0;
			}
		}

int ScreenUtilities::lock(SDL_Surface* surface)
{	
	if ( SDL_MUSTLOCK(surface) ) 
	{
		if ( SDL_LockSurface(surface) < 0 ) 
		{
			fprintf(stderr, "Can't lock screen: %s\n", SDL_GetError());
			return 1;
		}
	}
	return 0;
}


void ScreenUtilities::unlock(SDL_Surface* surface)
{
	if ( SDL_MUSTLOCK(surface) ) {
				SDL_UnlockSurface(surface);
			}
}


void ScreenUtilities::SaveScreenshot(SDL_Surface *screen)
{
	char c[128];
	int i, fd;

	// Check available file
	for(i = 0; i <= 999; i++)
	{
		if(i < 10) sprintf(c, "screenshot00%i.bmp", i); else
		if(i < 100) sprintf(c, "screenshot0%i.bmp", i); else
		sprintf(c, "screenshot%i.bmp", i);

		fd = open(c, O_CREAT | O_EXCL);
		if(fd) break;
	}

	// Save screenshot
	SDL_SaveBMP(screen, c);
}

void ScreenUtilities::decode_rgb(SDL_Surface* screen, int rgb, int *red, int *green, int *blue)
{
  // decide what to do depending on color depth
  switch(screen->format->BytesPerPixel)
  {
    case(1):
    {
      // 8 bits: 3 red, 3 green, 2 blue
      *red=((rgb>>5)&7)<<5;
      *green=((rgb>>2)&7)<<5;
      *blue=(rgb & 3)<<6;
      break;
    }
    case(2):
    {
      // 16 bits: 5 red, 6 green, 5 blue
      *red=((rgb>>11)&31)<<3;
      *green=((rgb>>5)&63)<<2;
      *blue=(rgb & 31)<<3;
      break;
    }
    case(3):
    {
      // 24 bits: 8 red, 8 green, 8 blue
      *red=(rgb>>16)&0xff;
      *green=(rgb>>8)&0xff;
      *blue=rgb & 0xff;
      break;
    }
    case(4):
    {
      // 32 bits: 8 red, 8 green, 8 blue
      *red=(rgb>>16)&0xff;
      *green=(rgb>>8)&0xff;
      *blue=rgb & 0xff;
      break;
    }
    default:
    {
      // unknown mode
      fprintf(stderr, "unknown bpp\n");
      *red=0;
      *green=0;
      *blue=0;
    }
  }
}

SDL_Surface* ScreenUtilities::drawText(SDL_Surface* bitmap, char* mytext, int X, int Y)
{
            FILE *file;
            char *myfont;
            myfont=(char *)malloc(1792);
            file = fopen(FONT_PATH,"r");
            fread(myfont,1792,1,file);
            fclose(file);
            
            static SDL_Surface *gfxPrimitivesFont[256];
            static int charWidth = 8, charHeight = 8;
            static int charPitch = 1;
            static int charSize = 8;
            
            charPitch = (charWidth+7)/8;
            charSize = charPitch * charHeight;
        
            for (int i = 0; i < 256; i++) {
            if (gfxPrimitivesFont[i]) {
                SDL_FreeSurface(gfxPrimitivesFont[i]);
                gfxPrimitivesFont[i] = NULL;}}
                
            stringRGBA(bitmap, X, Y, mytext, 0, 0, 0, 255);
            free(myfont);

    return bitmap;
}


